import cuffeo from 'cuffeo';
import { rankHands } from '@xpressit/winning-poker-hand-rank';
import handranker from './inc/handranker.mjs';
import cfg from '../config.json' assert { type: 'json' };

const bot = await new cuffeo(cfg.clients);

const suits = { "♠": "S", "♣": "C", "♦": "D", "♥": "H" };

const stripColors = msg => msg.replace(/\x03\d{0,2}(,\d{0,2}|\x02\x02)?/g, '');

const parseCards = msg => {
  const output = [];
  for(const c of stripColors(msg).match(/(\w+)([♠♣️♦♥️️])/g)) {
    if(c.length === 3)
      output.push(`T${suits[c[2]]}`);
    else
      output.push(`${c[0]}${suits[c[1]]}`);
  }
  return output;
};

const env = {
  hand: false,
  board: false,
  winchance: false,
  called: false,
  joined: false,
  callamount: 0,
  pot: 0,
  bank: 0
};

bot.on("message", e => {
  if(e.message.match(new RegExp(`^${e.self.me.nickname}: Your bank account`))) {
    env.bank = +e.message.match(/is: (\d+) \(/)[1];
  }
  if(e.message === ".pj" || e.message === "pj" && !env.hand && !env.joined) {
    env.joined = true;
    return e.reply("bb\npj");
  }
  if(e.message === ".pl" && !env.hand) {
    env.joined = false;
    return e.reply("pl");
  }
  if(e.message === ".bb" && !env.hand)
    return e.reply("bb");
  if(e.message === ".bl" && !env.hand)
    return e.reply("bank loan");
  if(e.message === ".s" && env.hand)
    return e.reply("s");
  if(e.message === ".f")
    return e.reply("f");
  if(e.message === ".d" && !env.hand)
    return e.reply("d");

  if(e.user.nick !== 'hirc' || !env.hand)
    return;
  
  if(e.message.match(/(small|big) blind/)) {
    env.callamount = +e.message.match(/pays (\d+) \(/)[1];
    env.pot += env.callamount;
    if(e.message.includes(e.self.me.nickname)) {
      env.bank -= +e.message.match(/pays (\d+) \(/)[1];
    }
    console.log(`callamount: ${env.callamount}; potsize: ${env.pot}; winchance: ${env.winchance}`);
  }
  if(e.message.includes('raises the pot')) {
    env.callamount = +e.message.match(/pot by (\d+)/)[1];
    env.pot += env.callamount;
    console.log(`callamount: ${env.callamount}; potsize: ${env.pot}; winchance: ${env.winchance}`);
  }
  if(e.message.match(/calls \d+\./)) {
    env.callamount = +e.message.match(/calls (\d+)\./)[1];
    env.pot += env.callamount;
    console.log(`callamount: ${env.callamount}; potsize: ${env.pot}; winchance: ${env.winchance}`);
  }
  if(e.message.includes('check')) {
    console.log(`callamount: ${env.callamount}; potsize: ${env.pot}; winchance: ${env.winchance}`);
  }
  
  if(e.message.match(new RegExp(`^Current player: ${e.self.me.nickname}( |$)`)) && env.hand) {
    if(!env.winchance) { // preflop
      const callby = +e.message.match(/\((\d+) to call\)/)?.[1] || 0;
      if(callby > 0) {
        env.bank -= callby;
      }
      return e.reply('c');
    }
    
    if(env.winchance >= 55 && env.callamount) {
      if(~~(Math.random() * 2) === 1) {
        const raiseby = env.callamount + (~~(Math.random() * 2 + 2) * 10);
        env.bank -= raiseby;
        return e.reply(`r ${raiseby}`);
      }
      else {
        return e.reply('c');
      }
    }

    if(e.message.includes('to call')) {
      if(env.winchance >= 30) {
        if(env.called) {
          env.bank -= env.callamount;
          return e.reply('c');
        }
        else {
          env.called = true;
          
          if((~~(Math.random() * 2)) === 1) {
            env.bank -= env.callamount;
            return e.reply('c');
          }
          else {
            return e.reply('f');
          }
        }
      }
      else {
        if((~~Math.random() * 5) === 1) {
          return e.reply('f');
        }
        else {
          env.bank -= env.callamount;
          return e.reply('c');
        }
      }
    }

    env.bank -= env.callamount;
    return e.reply('c');
  }
  if(e.message.includes('Game ended') || e.message.includes('wins the pot')) {
    //e.reply(`my hand: ${hand.join(', ')} (${winchance}%)`);
    e.reply('my bank balance: ' + env.bank);
    env.hand = false;
    env.board = false;
    env.winchance = false;
    env.called = false;
    env.joined = false;
    env.pot = 0;
    return;
  }

  if(e.message.match(/^(Flop|Turn|River)/) && e.user.nick === 'hirc') {
    env.board = parseCards(e.message);

    console.log(env.board, env.hand);

    const rank = rankHands('texas', env.board, [env.hand])[0];
    env.winchance = (9999 - rank.rank) / 100;
  }
});

bot.on("notice", msg => {
  if(msg.match(/^Your hand/)) {
    env.hand = parseCards(msg);
  }
});
